package RDF::Schema::Model;
use strict;
use Carp;
use Time::HiRes qw( time );

use RDF::Simple::Model;
use RDF::Schema::Resource;
use RDF::Schema::Statement;
use vars qw( @ISA );

@ISA = qw( RDF::Simple::Model );


sub ns_rdfs                          { 'rdfs:' }
sub rdfs
{ 
    my $node = $_[0]->{'cache'}{'node'}{&ns_rdfs.$_[1]};
    unless( $node )
    {
	$node = $_[0]->get_node_by_uri( &ns_rdfs.$_[1] );
	$_[0]->{'cache'}{'node'}{&ns_rdfs.$_[1]} = $node;
    }
    return $node;
}


sub get_top_classes
{
    my( $self ) = @_;

    my @sub_list = ();
    foreach my $sub (@{ $self->rdfs('Class')->get_subjects( $self->rdf('type') ) })
    {
	push @sub_list, $sub unless 
	    $sub->has_property( $self->rdfs('subClassOf') );
    }
    return \@sub_list;
}

sub get_subclasses_for
{
    my( $self, $class ) = @_;

    my $subclasses = $class->get_subjects( $self->rdfs('subClassOf') );

    my @subsubs;
    foreach my $class ( @$subclasses )
    {
	push @subsubs, @{ $self->get_subclasses_for( $class ) };
    }
    push @$subclasses, @subsubs;

    return $subclasses;
}

sub get_nodes_of_type
{
    my( $self, $type, $option ) = @_;

    my $order = $option->{'order'}||'';

    # This cache cuts the time from 1.0 to 0.003 secs
    # The cahce must time out or update at some point
    #
    my $nodes_of_type = $self->{'cache'}{'nodes_of_type'}{$type->uri}{$order};
    unless( $nodes_of_type )
    {
	if( $type->uri eq $self->rdfs('Resource')->uri )
	{
	    return $self->get_some_Resources;
	}

#	warn $type->uri."\n";
#	my $time = time; ##

	my %nodes;
	my $subclasses = $self->get_subclasses_for( $type );
	foreach my $class ( $type, @$subclasses )
	{
	    foreach my $node ( @{ $self->get_nodes_of_type_nonrecursively( $class ) } )
	    {
		$nodes{ $node->uri } ||= $node;
	    }
#	    warn "\t".(time-$time)."\n";
	}
	if( $order eq 'by_label' )
	{
	    warn "by_label\n";
	    $nodes_of_type = [ sort by_label values %nodes ];
	}
	else
	{
	    warn "random\n";
	    $nodes_of_type = [ values %nodes ];
	}
	$self->{'cache'}{'nodes_of_type'}{$type->uri}{$order} = $nodes_of_type;
    } 
    return $nodes_of_type;
}

sub by_label { ($a->label||$a->uri) cmp ($b->label||$b->uri) }

sub get_nodes_of_type_nonrecursively
{
    my( $self, $type ) = @_;

#    warn "T: ".$type->uri."\n";

    if( $type->uri eq $self->rdfs('Resource')->uri )
    {
	my @lone_resources;
	foreach my $res ( @{ $self->get_some_Resources } )
	{
	    push @lone_resources, $res unless
		$res->has_property( $self->rdf('type') )
		    or $res->is_instance_of( $self->rdfs('ContainerMembershipProperty') );
	}
	return \@lone_resources;
    }
    elsif( $type->uri eq $self->rdfs('Literal')->uri )
    {
	return $self->get_Literals;
    }
    elsif( $type->uri eq $self->rdfs('ContainerMembershipProperty')->uri )
    {
	return $self->get_ContainerMembershipProperties;
    }
    return $type->get_subjects( $self->rdf('type') );
}



sub new_node
{
    my( $self ) = @_;
    return new RDF::Schema::Resource( $self );
}

sub new_arc
{
    my( $self ) = @_;
    return new RDF::Schema::Statement( $self );
}

1;





