package RDF::Simple::Model;
use strict;
use CGI::Carp;
use vars qw( @ISA );
@ISA = qw( );  # Defined by constructor

use RDF::Simple::Resource;
use RDF::Simple::Statement;

sub ns_rdf    { 'rdf:' }
sub ns_local  { ':' }
sub rdf
{ 
    my $node = $_[0]->{'cache'}{'node'}{&ns_rdf.$_[1]};
    unless( $node )
    {
	$node = $_[0]->get_node_by_uri( &ns_rdf.$_[1] );
	$_[0]->{'cache'}{'node'}{&ns_rdf.$_[1]} = $node;
    }
    return $node;
}

sub new
{
    my( $class, $source, @connect ) = @_;

    defined $source or confess "Source missing";
    my $source_mod  = "RDF::Source::".$source."::Model";

    my $source_file = $source_mod.".pm";
    $source_file =~ s|::|/|g;

    eval {require $source_file} or croak $@;
    push @ISA, $source_mod;
    my $self = 
    {
	source  =>
	{
	    name => $source,
	},
    };
    bless ($self, $class);

    $self->SUPER::init( @connect );
    return $self;
}

sub init
{
    my( $self ) = @_;
    my $source = $self->{'source'}{'name'};
    croak "No initialization defined by '$source'";
}

sub new_node
{
    my( $self ) = @_;
    return new RDF::Simple::Resource( $self );
}

sub new_arc
{
    my( $self ) = @_;
    return new RDF::Simple::Statement( $self );
}

sub get_nodes_of_type
{
    my( $self, $type ) = @_;

    return $type->get_subjects( $self->rdf('type') );
}



1;
